package com.shibofu.spring.db3.serviceImpl;

import com.shibofu.spring.db1.dao.ArchiveDetailDao;
import com.shibofu.spring.db1.dao.ArchiveMasterDao;
import com.shibofu.spring.db3.dao.CostListDao;
import com.shibofu.spring.db3.service.CostListService;
import com.shibofu.spring.dto.ArchiveDetailDto;
import com.shibofu.spring.vo.ArchiveDetailVo;
import com.shibofu.spring.vo.CostListVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ClassName CostListServiceImpl
 * @Description 费用清单
 * @Author linjj
 * @Date 2024/5/22 10:19
 * @Version 1.0
 */
@Service
@Slf4j
public class CostListServiceImpl implements CostListService {

    @Autowired
    private CostListDao costListDao;
    @Autowired
    private ArchiveMasterDao archiveMasterDao;
    @Autowired
    private ArchiveDetailDao archiveDetailDao;
    @Value("${savePath}")
    private String savePath;


    /**
     * @description: 采集费用清单
     * @author linjj
     * @date: 2024/5/22 10:21
     */
    @Override
    public void collectionCostList() {
        //插入文件表数据集合
        List<ArchiveDetailDto> ArchiveDetailList = new ArrayList<>();
        //查询一共有多条记录
        int conut = costListDao.getConut();
        //每次查询50条记录
        int pollingNum = (int) Math.floor((double) conut / (double) 50);
        for (int i = 0; i <= pollingNum; i++) {
            //每次查询50条记录
            int offset = i * 50;
            //查询需要采集清单病历
            List<CostListVo> costList = costListDao.getCostList(offset);
            if (!CollectionUtils.isEmpty(costList)) {
                log.info("费用清单本次采集" + costList.size() + "份!!!");
                for (CostListVo list : costList) {
                    if (StringUtils.isEmpty(list.getFpdf())) {
                        log.info(list.getFjzh() + "路径为空！！");
                        continue;
                    }
                    try {
                        download(list, ArchiveDetailList);
                    } catch (IOException e) {
                        log.info(e.getMessage());
                    }
                }
                archiveDetailDao.addArchiveDetail(ArchiveDetailList);
                log.info("完成采集" + ArchiveDetailList.size() + "份!!!");
                ArchiveDetailList.clear();
            }
        }
        log.info("本次采集没有费用清单记录");
    }

    private void download(CostListVo costListVo, List<ArchiveDetailDto> ArchiveDetailList) throws IOException {
        ArchiveDetailDto archiveDetailDto = new ArchiveDetailDto();
        //使用yyyyMMddHHmmssSSS格式作为文件名
        Date date = new Date();
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmssSSS");
        String newDate = format.format(date);
        //根据jzh查询患者主键id
        String masterId = archiveMasterDao.getIdbyJzh(costListVo.getFjzh());
        //判断文件是否存在存在删除后新增不存在新增
        List<ArchiveDetailVo> subAssort = archiveDetailDao.getSubAssort(masterId);
        if (subAssort.size() > 0) {
            deleteFliepath(subAssort);
        }
        //组织保存目录
        String filePathdir = savePath + File.separatorChar + costListVo.getFpatid() + File.separatorChar + costListVo.getFtimes();
        File file = new File(filePathdir);
        //判断文件夹是否存在不存在创建文件夹
        if (!file.exists()) {
            file.mkdirs();
        }
        //文件保存路径
        String filePath = filePathdir + File.separatorChar + newDate + ".pdf";
        //下载文件
        try (FileOutputStream fileOutputStream = new FileOutputStream(filePath)) {
            // 将二进制数据写入文件
            fileOutputStream.write(costListVo.getFpdf());
            fileOutputStream.flush();
        } catch (IOException e) {
            log.info(masterId + "异常处理" + e.getMessage());
        }
        //保存文件表
        archiveDetailDto.setMasterId(masterId);
        archiveDetailDto.setUploadDateTime(new Date());
        archiveDetailDto.setAssortId("DE599D770E8347CCB5122BC357D96F47");
        archiveDetailDto.setSource("pacs");
        archiveDetailDto.setFlag("0");
        archiveDetailDto.setTitle("费用结算清单");
        archiveDetailDto.setPdfPath(filePath);
        archiveDetailDto.setSubAssort(masterId);
        ArchiveDetailList.add(archiveDetailDto);
    }


    private void deleteFliepath(List<ArchiveDetailVo> listPath) {
        for (ArchiveDetailVo list : listPath) {
            archiveDetailDao.delSubAssort(list.getId());
            File file = new File(list.getPdfPath());
            try {
                file.delete(); // 删除照片
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

}
